
const int interruptPin = 2;
const int PIN_CHNUP = 4;
const int PIN_CHNDN = 5;
const int PIN_VOLUP = 6;
const int PIN_VOLDN = 7;
const int PIN_PHONE = 8;
const int PIN_RT = 9;
const int PIN_SPUP = 10;
const int PIN_SPDN = 16;
const int PIN_CC = 14;
const int PIN_IO = 15;

boolean m_Possibles[12];
byte Events[12][11]= {{0, 0x86, 0x1E, 0x80, 0xF8, 0x06, 0x98, 0x98, 0xff, 0xff, 0xff},
                      {0, 0x86, 0x60, 0x80, 0x9E, 0x18, 0x78, 0xE0, 0xF8, 0x80, 0xff},
                      {0, 0x86, 0x60, 0x80, 0x9E, 0x18, 0x78, 0x60, 0xFE, 0xE0, 0xff},
                      {0, 0x86, 0x60, 0x80, 0x9E, 0x9E, 0x78, 0x80, 0xFE, 0x00, 0xff},
                      {0, 0x86, 0x60, 0x80, 0x9E, 0x9E, 0x78, 0x80, 0xFE, 0xE0, 0xff},
                      {0, 0x86, 0x60, 0x80, 0x9E, 0x9E, 0x78, 0x06, 0xFE, 0x86, 0xff},
                      {0, 0x86, 0x60, 0x80, 0x9E, 0x9E, 0x78, 0x00, 0x78, 0xff, 0xff},
                      {0, 0x86, 0x60, 0x80, 0x9E, 0x9E, 0x78, 0x60, 0x78, 0xE0, 0xff},
                      {0, 0x86, 0x60, 0x80, 0x9E, 0x9E, 0x78, 0x80, 0x60, 0x18, 0xff},
                      {0, 0x86, 0x60, 0x80, 0xF8, 0x9E, 0x78, 0x7E, 0xE0, 0xff, 0xff},
                      {0, 0x86, 0x60, 0x80, 0xF8, 0x9E, 0x78, 0x98, 0x7E, 0xF8, 0xff},
                      {0, 0x86, 0x60, 0x80, 0xF8, 0x9E, 0x78, 0x66, 0x00, 0xff, 0xff}};
int m_Pointer=0;

enum eEvents{
  eRT,
  eVolPlus,
  eVolMinus,
  eChPlusDown,
  eChPlusLong,
  eChPlusUp,
  eChMinusDown,
  eChMinusLong,
  eChMinusUp,
  ePhoneDown,
  ePhoneLong,
  ePhoneUp,
};

long m_EndTime;
long m_Duration=100;
int INTERRUPTPIN;
int m_CCReceivedBuffer;
volatile int m_CCReceived;
volatile bool m_CCNewData;


void PinInterrupt(){
  delayMicroseconds(234);
  int bit = digitalRead(interruptPin);
  delayMicroseconds(234);
  if(digitalRead(interruptPin)==HIGH){
    m_CCReceived=m_CCReceivedBuffer>>1;
    m_CCReceivedBuffer=0;
    m_CCNewData = true;
  }else{
    m_CCReceivedBuffer = (m_CCReceivedBuffer<<1) + bit;
  }
}


void setup() {
  Serial.begin(9600);
  Serial1.begin(19200);
  delay(50);
  Serial.println("Start");
  for(int i=0;i<12;i++)m_Possibles[i]=true;
  m_Pointer=0;
  m_EndTime=0;
  m_CCReceived=0;
  m_CCReceivedBuffer=0;
  m_CCNewData=false;

  pinMode(PIN_CHNDN, OUTPUT);
  pinMode(PIN_CHNUP, OUTPUT);
  pinMode(PIN_VOLDN, OUTPUT);
  pinMode(PIN_VOLUP, OUTPUT);
  pinMode(PIN_PHONE, OUTPUT);
  pinMode(PIN_RT, OUTPUT);
  pinMode(PIN_CC, OUTPUT);
  pinMode(PIN_IO, OUTPUT);
  pinMode(PIN_SPDN, OUTPUT);
  pinMode(PIN_SPUP, OUTPUT);
  pinMode(interruptPin, INPUT);
  INTERRUPTPIN=digitalPinToInterrupt(interruptPin);
  attachInterrupt(INTERRUPTPIN, PinInterrupt, RISING);
}

void loop() {
  if(m_EndTime != 0 && millis()>m_EndTime){
    digitalWrite(PIN_RT, LOW);
    digitalWrite(PIN_VOLUP, LOW);
    digitalWrite(PIN_VOLDN, LOW);
    m_EndTime=0;
  } 
  if (Serial1.available()>0) readRadio();
  if (m_CCNewData) readCC();
}


void readRadio(){
  int ret=Serial1.read();
  switch(m_Pointer){
    case 0:
      m_Pointer = ret == 0? m_Pointer + 1 : 0;
    break;
    case 1:
      m_Pointer = ret == 0x86? m_Pointer + 1 : 0;
    break;
    default:
      m_Pointer= CheckPointer(ret)? m_Pointer + 1 : 0;
    break;
  }
  if(m_Pointer>0)return;
  for(int i=0;i<12;i++)m_Possibles[i]=true;
}

boolean CheckPointer(int ret){
  int found=-1;
  for(int e=0;e<12;e++){
    if(!m_Possibles[e])continue;

    if(Events[e][m_Pointer]==ret){
      found=e;
    }else{
      m_Possibles[e]=false;
    }
  }
  if(found<0) return false;
  if(Events[found][m_Pointer+1] != 0xFF)return true;
 
  RaiseEvent(found);
  return false;
}

void RaiseEvent(int event){
  Serial.println(event);
  switch(event){
    case eRT:
      m_EndTime = millis()+m_Duration;
      digitalWrite(PIN_RT, HIGH);
    break;

    case eVolPlus:
      m_EndTime = millis()+m_Duration;
      digitalWrite(PIN_VOLUP, HIGH);
    break;

    case eVolMinus:
      m_EndTime = millis()+m_Duration;
      digitalWrite(PIN_VOLDN, HIGH);
    break;

    case eChPlusDown:
      digitalWrite(PIN_CHNUP, HIGH);
    break;

    case eChMinusDown: 
      digitalWrite(PIN_CHNDN, HIGH);
    break;

    case ePhoneDown:
      digitalWrite(PIN_PHONE, HIGH);
    break;

    case eChPlusUp:
      digitalWrite(PIN_CHNUP, LOW);
    break;

    case eChMinusUp:
      digitalWrite(PIN_CHNDN, LOW);
    break;

    case ePhoneUp:
      digitalWrite(PIN_PHONE, LOW);
    break;
  }
}

void readCC(){
  m_CCNewData=false;
  switch(m_CCReceived){
    case 127:
      digitalWrite(PIN_CC, LOW);
      digitalWrite(PIN_IO, HIGH);
      digitalWrite(PIN_SPUP, LOW);
      digitalWrite(PIN_SPDN, LOW);
    break;

    case 55:
      digitalWrite(PIN_CC, HIGH);
    break;

    case 109:
      digitalWrite(PIN_IO, LOW);
    break;

    case 91:
      digitalWrite(PIN_SPUP, HIGH);
    break;

    case 126:
      digitalWrite(PIN_SPDN, HIGH);
    break;
  }
  m_CCReceived=0;
}
